// University of Washington Bothell Graphics Library
// Author: Timothy Chuang
// Based on Kelvin Sung and Steve Baer's D3D Library 
// and Robert Stone's XNA Library version 5
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

namespace MagGame
{

    public abstract class UWB_BoundingVolume
    {
        public enum volume_type { box, sphere, circle };
        /// Return the type of bounding volume this class is
        /// This is a very simple type of runtime type information
        public abstract volume_type getType();

        ///
        /// Return the center of the volume
        public abstract Vector3 GetCenter();

        /// Union another volume with this volume to make this volume potentially larger
        /// \param pBV the other volume to union with this
        public abstract void Add(UWB_BoundingVolume pBV);

        public abstract bool isValid(); // do min/max points form a valid rectangle

        /// Test if another volume shares space with this volume
        /// \return true if an intersection exists
        public abstract bool Intersects(ref UWB_BoundingVolume other);

        /// Test if a given point is inside of this volume
        /// \return true if test_point is in the space that this volume contains
        public abstract bool ContainsPoint(Vector3 test_point);

        /// Draw this volume with a draw helper class
        public abstract void Draw(UWB_DrawHelper pDrwHelper);

        public static bool UWB_IntersectBoxBox(Vector3 box1_min, Vector3 box1_max,
                                  Vector3 box2_min, Vector3 box2_max)
        {
            // an intersection only exists if the boxes have some sort of overlap
            // along all three axes. Just test each axis individually
            bool xOutside = box1_min.X > box2_max.X || box1_max.X < box2_min.X;
            if (xOutside)
                return false;

            bool yOutside = box1_min.Y > box2_max.Y || box1_max.Y < box2_min.Y;
            if (yOutside)
                return false;

            bool zOutside = box1_min.Z > box2_max.Z || box1_max.Z < box2_min.Z;
            if (zOutside)
                return false;

            return true;
        }

        public static bool UWB_IntersectCircleCircle(Vector3 c1, float r1, Vector3 c2, float r2)
        {
            float distance = Vector3.Distance(c1, c2);
            if( distance <= (r1 + r2) )
                return true;
            else
                return false;

        }
    }

    public class UWB_BoundingBox : UWB_BoundingVolume
    {
        private Vector3 m_min;
        private Vector3 m_max;

        public UWB_BoundingBox()
        {
            m_min = new Vector3(0, 0, 0);
            m_max = new Vector3(0, 0, 0);
        }

        public UWB_BoundingBox(Vector3 corner1, Vector3 corner2)
        {
            m_min = corner1;
            m_max = corner2;
        }



        public UWB_BoundingBox(Vector3 center, float width, float height, float depth)
        {
            m_min.X = center.X - width / 2.0f;
            m_min.Y = center.Y - height / 2.0f;
            m_min.Z = center.Z - depth / 2.0f;

            m_max.X = center.X + width / 2.0f;
            m_max.Y = center.Y + height / 2.0f;
            m_max.Z = center.Z + depth / 2.0f;
        }

        public override bool isValid()
        {
            return (m_min.X <= m_max.X && m_min.Y <= m_max.Y && m_min.Z <= m_max.Z);
        }

        public override volume_type getType()
        {
            return volume_type.box;
        }

        public override Vector3 GetCenter()
        {
            return (m_max + m_min) / 2.0f;
        }

        public override void Add(UWB_BoundingVolume pBV)
        {
            volume_type bt = pBV.getType();

            if (volume_type.box == bt)
            {
                UWB_BoundingBox box = (UWB_BoundingBox)pBV;
                if (!box.isValid())
                    return;
                if (!isValid())
                {
                    m_min = box.GetMin();
                    m_max = box.GetMax();
                }
                else
                {
                    m_min.X = m_min.X < box.m_min.X ? m_min.X : box.m_min.X;
                    m_min.Y = m_min.Y < box.m_min.Y ? m_min.Y : box.m_min.Y;
                    m_min.Z = m_min.Z < box.m_min.Z ? m_min.Z : box.m_min.Z;
                    m_max.X = m_max.X > box.m_max.X ? m_max.X : box.m_max.X;
                    m_max.Y = m_max.Y > box.m_max.Y ? m_max.Y : box.m_max.Y;
                    m_max.Z = m_max.Z > box.m_max.Z ? m_max.Z : box.m_max.Z;
                }

            }
        }

        public override bool Intersects(ref UWB_BoundingVolume other)
        {
            volume_type bt = other.getType();

            if (volume_type.box == bt)
            {
                UWB_BoundingBox box = (UWB_BoundingBox)other;
                return UWB_IntersectBoxBox(m_min, m_max, box.GetMin(), box.GetMax());
            }
            //If we get here, we are intersecting with an unknown type
            return false;
        }

        public override bool ContainsPoint(Vector3 location)
        {
            // test along each axis to make sure the point is inside this volume
            bool xInside = location.X >= m_min.X && location.X <= m_max.X;
            if (!xInside)
                return false;

            bool yInside = location.Y >= m_min.Y && location.Y <= m_max.Y;
            if (!yInside)
                return false;

            bool zInside = location.Z >= m_min.Z && location.Z <= m_max.Z;
            if (!zInside)
                return false;

            return true;
        }

        public override void Draw(UWB_DrawHelper pDrawHelper)
        {
            if (pDrawHelper != null)
            {
                Vector3 start = new Vector3(m_min.X, m_min.Y, m_min.Z);
                Vector3 end = new Vector3(m_min.X, m_min.Y, m_min.Z);
                end.X = m_max.X;
                pDrawHelper.DrawLine(start, end);
                start = end;
                end.Y = m_max.Y;
                pDrawHelper.DrawLine(start, end);
                start = end;
                end.X = m_min.X;
                pDrawHelper.DrawLine(start, end);
                start = end;
                end.Y = m_min.Y;
                pDrawHelper.DrawLine(start, end);
            }
        }

        public void SetCorners(Vector3 corner1, Vector3 corner2)
        {
            // test each point to determine which as max/min
            m_min.X = corner1.X < corner2.X ? corner1.X : corner2.X;
            m_min.Y = corner1.Y < corner2.Y ? corner1.Y : corner2.Y;
            m_min.Z = corner1.Z < corner2.Z ? corner1.Z : corner2.Z;

            m_max.X = corner1.X > corner2.X ? corner1.X : corner2.X;
            m_max.Y = corner1.Y > corner2.Y ? corner1.Y : corner2.Y;
            m_max.Z = corner1.Z > corner2.Z ? corner1.Z : corner2.Z;
        }

        public Vector3 GetMin()
        {
            return m_min;
        }

        public Vector3 GetMax()
        {
            return m_max;
        }

        public void MakeInvalid()
        {
            //make min > max
            m_max = new Vector3(-1, -1, -1);
            m_min = new Vector3(1, 1, 1);
        }

        ///
        /// Return the length along the x-axis that this box takes up
        public float Width()
        {
            return Math.Abs(m_max.X - m_min.X);
        }
        ///
        /// Return the length along the y-axis that this box takes up
        public float Height()
        {
            return Math.Abs(m_max.Y - m_min.Y);
        }
        ///
        /// Return the length along the z-axis that this box takes up
        public float Depth()
        {
            return Math.Abs(m_max.Z - m_min.Z);
        }

    }

    public class UWB_BoundingCircle : UWB_BoundingVolume
    {
        private Vector3 m_center;
        private float m_radius;

        public UWB_BoundingCircle()
        {
            m_center = new Vector3(0, 0, 0);
            m_radius = 0.0f;
        }

        public UWB_BoundingCircle(Vector3 c, float r)
        {
            m_center = c;
            m_radius = r;
        }

        public override volume_type getType()
        {
            return volume_type.circle;
        }

        public override Vector3 GetCenter()
        {
            return m_center;
            //throw new Exception("The method or operation is not implemented.");
        }
        ///
        /// Return the center of the volume
        public Vector3 Center
        {
            get
            {
                return m_center;
            }
            set
            {
                m_center = value;
            }
        }

        public float Radius
        {
            get
            {
                return m_radius;
            }
            set
            {
                m_radius = value;
            }
        }

        /// Union another volume with this volume to make this volume potentially larger
        /// \param pBV the other volume to union with this
        public override void Add(UWB_BoundingVolume pBV)
        {
            throw new Exception("The method or operation is not implemented.");
        }


        public override bool isValid()
        {
            return (m_radius >= 0.0f);
        }


        /// Test if another volume shares space with this volume
        /// \return true if an intersection exists
        public override bool Intersects(ref UWB_BoundingVolume other)
        {
            if (other.getType() == volume_type.circle)
            {
                UWB_BoundingCircle circle = (UWB_BoundingCircle)other;
                return UWB_IntersectCircleCircle(m_center, m_radius, circle.Center, circle.Radius);
            }
            //If we get here, we are intersecting with an unknown type
            return false;
        }

        /// Test if a given point is inside of this volume
        /// \return true if test_point is in the space that this volume contains
        public override bool ContainsPoint(Vector3 location)
        {
            bool isInside = Vector3.Distance(location, m_center) > m_radius ? false : true;
            
            return isInside;
        }

        /// Draw this volume with a draw helper class
        public override void Draw(UWB_DrawHelper pDrwHelper)
        {
            if (pDrwHelper != null)
                pDrwHelper.DrawCircle(m_center, m_radius);
        }

    }
}
