// University of Washington Bothell Graphics Library
// Author: Howard Gene Ramsdell
// Based on Kelvin Sung and Steve Baer's D3D Library 
// and Robert Stone's XNA Library version 5
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MagGame
{
    public class UWB_SceneNode
    {
        // Members
        private string m_Name;
        private bool m_bPivotVisible;
        private UWB_Primitive m_pPrimitive;
        private UWB_XformInfo m_xform;
        private UWB_Array<UWB_SceneNode> m_aChildNodes;
        private Vector3 m_vVelocity;
        private Vector3 originalPivotPos; // BADD!!!!
        public Vector3 originalPivot
        {
            get { return originalPivotPos; }
            set { originalPivotPos = value; }
        }

        // Constructor(s)
        public UWB_SceneNode(string name)
        {
            m_Name = name;
            m_bPivotVisible = false;
            m_pPrimitive = null;
            m_aChildNodes = new UWB_Array<UWB_SceneNode>();
            m_xform = new UWB_XformInfo();
            m_vVelocity = new Vector3(0, 0, 0);
        }


        // Public Methods
        public void draw(eLevelofDetail lod, UWB_DrawHelper drawHelper)
        {
            drawHelper.PushModelTransform();
            m_xform.SetUpModelStack(drawHelper);

            int count = m_aChildNodes.count();
            for (int i = 0; i < count; i++)
            {
                m_aChildNodes.getItem(i).draw(lod, drawHelper);
            }

            if (m_pPrimitive != null)
            {
                m_pPrimitive.draw(lod, drawHelper);
            }

            if (m_bPivotVisible)
            {
                m_xform.DrawPivot(drawHelper, 0.2f);
            }

            drawHelper.PopModelTransform();
        }

        public void setPrimitive(UWB_Primitive primitive)
        {
            m_pPrimitive = primitive;
        }

        public UWB_Primitive getPrimitive()
        {
            return m_pPrimitive;
        }

        public void insertChildNode(UWB_SceneNode node)
        {
            m_aChildNodes.append(node);
        }

        public UWB_SceneNode getChildNode(int index)
        {
            if (index < 0 || index >= numChildren())
                return null;

            return m_aChildNodes.getItem(index);
        }

        public int numChildren()
        {
            return m_aChildNodes.count();
        }

        public void moveNodeByVelocity(float elapsed_seconds)
        {
            Vector3 translate = m_xform.GetTranslation();
            translate += (m_vVelocity * elapsed_seconds);
            m_xform.SetTranslation(translate);
        }

        public void setVelocity(Vector3 velocity)
        {
            m_vVelocity = velocity;
        }

        public Vector3 getVelocity()
        {
            return m_vVelocity;
        }

        public UWB_XformInfo getXformInfo()
        {
            return m_xform;
        }

        public void setXformInfo(UWB_XformInfo xf)
        {
            m_xform = xf;
        }

        public void setPivotVisible(bool on)
        {
            m_bPivotVisible = on;
        }

        public bool IsPivotVisible()
        {
            return m_bPivotVisible;
        }

        public UWB_BoundingBox getBounds(UWB_DrawHelper drawHelper, bool bDraw)
        {
            UWB_BoundingBox box = new UWB_BoundingBox();
            box.MakeInvalid();
            if (m_pPrimitive != null)
            {
                box.Add(m_pPrimitive.GetBoundingVolume(drawHelper.getLod()));
            }

            drawHelper.PushModelTransform();
            m_xform.SetUpModelStack(drawHelper);

            // Draw the box for debugging
            if (bDraw)
            {
                box.Draw(drawHelper);
            }

            Vector3 minPt = box.GetMin();
            Vector3 maxPt = box.GetMax();
            Vector3 pt1 = new Vector3(minPt.X, minPt.Y, minPt.Z);
            Vector3 pt2 = new Vector3(maxPt.X, minPt.Y, minPt.Z);
            Vector3 pt3 = new Vector3(maxPt.X, maxPt.Y, minPt.Z);
            Vector3 pt4 = new Vector3(minPt.X, maxPt.Y, minPt.Z);
            Vector3 pt5 = new Vector3(minPt.X, minPt.Y, maxPt.Z);
            Vector3 pt6 = new Vector3(maxPt.X, minPt.Y, maxPt.Z);
            Vector3 pt7 = new Vector3(maxPt.X, maxPt.Y, maxPt.Z);
            Vector3 pt8 = new Vector3(minPt.X, maxPt.Y, maxPt.Z);
            drawHelper.TransformPoint(ref pt1);
            drawHelper.TransformPoint(ref pt2);
            drawHelper.TransformPoint(ref pt3);
            drawHelper.TransformPoint(ref pt4);
            drawHelper.TransformPoint(ref pt5);
            drawHelper.TransformPoint(ref pt6);
            drawHelper.TransformPoint(ref pt7);
            drawHelper.TransformPoint(ref pt8);

            box.MakeInvalid();
            box.Add(new UWB_BoundingBox(pt1, pt2));
            box.Add(new UWB_BoundingBox(pt3, pt4));
            box.Add(new UWB_BoundingBox(pt5, pt6));
            box.Add(new UWB_BoundingBox(pt7, pt8));

            int count = numChildren();
            for (int i = 0; i < count; i++)
            {
                box.Add(m_aChildNodes.getItem(i).getPrimitive().GetBoundingVolume(drawHelper.getLod()));
            }

            drawHelper.PopModelTransform();
            return box;
        }

        public UWB_SceneNode searchNodeByName(string name)
        {
            if (name == this.m_Name)
                return this;
            int count = m_aChildNodes.count();
            
            for (int i = 0; i < count; i++)
            {
                UWB_SceneNode searchNode = m_aChildNodes.getItem(i).searchNodeByName(name);
                if (searchNode != null)
                    return searchNode;
            }
            return null;
        }

        public bool getNodeBoundsHelper(UWB_SceneNode searchNode,
            UWB_BoundingBox box,
            UWB_DrawHelper drawHelper,
            int level,
            bool bDraw)
        {
            bool found = false;

            if (this == searchNode)
            {
                box = this.getBounds(drawHelper, bDraw);
                found = true;
            }
            else
            {
                drawHelper.PushModelTransform();
                level++;
                this.getXformInfo().SetUpModelStack(drawHelper);
                int count = this.numChildren();

                for (int i = 0; i < count; i++)
                {
                    UWB_SceneNode child = this.getChildNode(i);
                    if (child.getNodeBoundsHelper(searchNode, box, drawHelper, level, bDraw))
                    {
                        found = true;
                        break;
                    }
                }
                level--;
                drawHelper.PopModelTransform();
            }

            if (level == 0 && found && bDraw)
            {
                box.Draw(drawHelper);
            }

            return found;
        }

        public bool getNodeBounds(UWB_SceneNode searchNode,
            UWB_BoundingBox box,
            UWB_DrawHelper drawHelper,
            bool bDraw)
        {
            return getNodeBoundsHelper(searchNode, box, drawHelper, 0, bDraw);
        }
    }
}