// University of Washington Bothell Graphics Library
// Modified by: Timothy Chuang
// Based on Kelvin Sung and Steve Baer's D3D Library 
// and Robert Stone's XNA Library version 5
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

namespace MagGame
{
    public class UWB_XNADrawHelper : UWB_DrawHelper
    {
        private UWB_XNAD3DMatrixStack m_pMatrixStack;

        public UWB_XNADrawHelper()
        {
            m_pMatrixStack = new UWB_XNAD3DMatrixStack();
        }

        public override bool DrawPoint(Vector3 position)
        {
            GraphicsDevice device = MagGame.MyGraphicsManager.GraphicsDevice;
			Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");

			DeviceVertexFormat[] v = new DeviceVertexFormat[1];
			v[0].Position = position;
			v[0].Color = mColor1;
            v[0].U = 0.0f;
            v[0].V = 0.0f;

			float pointSize = device.RenderState.PointSize;
			device.RenderState.PointSize = mPointSize;

			device.DrawUserPrimitives(PrimitiveType.PointList, v, 0, 1);

            return true;
        }

        public override bool DrawLine(Vector3 start, Vector3 end)
        {
			GraphicsDevice device = MagGame.MyGraphicsManager.GraphicsDevice;
			Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");

			DeviceVertexFormat[] v = new DeviceVertexFormat[2];

			Color flatColor = mColor1;

			v[0].Position = start;
			v[0].Color = flatColor;
            v[0].U = 0.0f;
            v[0].V = 0.0f;

            v[1].Position = end;
            v[1].U = 1.0f;
            v[1].V = 1.0f;
			if (mShadeMode == eShadeMode.smFlat)
				v[1].Color = flatColor;
			else
				v[1].Color = mColor2;

			device.DrawUserPrimitives(PrimitiveType.LineList, v, 0, 1);

            return true;
        }

        public override bool DrawRectangleXY(Vector3 min, Vector3 max)
        {
            GraphicsDevice device = MagGame.MyGraphicsManager.GraphicsDevice;
            Debug.Assert(device != null, "device not valid on DrawHelper drawRectangle call");

            DeviceVertexFormat[] v = new DeviceVertexFormat[6];
            Color flatColor = mColor1;


            v[0].Position = Vector3.Divide((min + max), 2);
            v[0].Color = flatColor;
            v[0].U = 0.5f;
            v[0].V = 0.5f;

            v[1].Position = new Vector3(min.X, max.Y, max.Z);
            v[1].Color = flatColor;
            v[1].U = 1.0f;
            v[1].V = 0.0f;

            v[2].Position = min;
            v[2].Color = flatColor;
            v[2].U = 1.0f;
            v[2].V = 1.0f;

            v[3].Position = new Vector3(max.X, min.Y, max.Z);
            v[3].Color = flatColor;
            v[3].U = 0.0f;
            v[3].V = 1.0f;

            v[4].Position = max;
            v[4].Color = flatColor;
            v[4].U = 0.0f;
            v[4].V = 0.0f;

            v[5] = v[1];

            if (mShadeMode == eShadeMode.smGouraud)
                v[0].Color = mColor2;

            device.DrawUserPrimitives(PrimitiveType.TriangleFan, v, 0, 4);
            return true;
        }


        public override bool DrawCircle(Vector3 center, float radius)
        {
			GraphicsDevice device = MagGame.MyGraphicsManager.GraphicsDevice;
            Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");
			if (radius > 0)
			{
				int kNumPts = 40;
				if (mLod == eLevelofDetail.lodLow)
					kNumPts = 10;
				else if (mLod == eLevelofDetail.lodMed)
					kNumPts = 20;
				DeviceVertexFormat[] v = new DeviceVertexFormat[kNumPts];

				float delta = (float)(UWB_Math.UWB_ToRadians((float)360 / (float)(kNumPts-2)));

                Color flatColor = Color.Pink; //mColor1;

				v[0].Position = new Vector3(center.X, center.Y, 0f);
				v[0].Color = flatColor;
                v[0].U = 0.5f;
                v[0].V = 0.5f;

				for (int i = 0; i < (kNumPts - 2); i++)
				{
					float X = center.X + (radius * (float)Math.Cos(i * delta));
					float Y = center.Y + (radius * (float)Math.Sin(i * delta));
					v[i + 1].Position = new Vector3(X, Y, 0f);
					v[i + 1].Color = flatColor;
                    v[i + 1].U = 0.5f + (0.5f * (float)Math.Cos(i * delta));
                    v[i + 1].V = 0.5f - (0.5f * (float)Math.Sin(i * delta));
				}
				v[kNumPts - 1] = v[1];

				if(mShadeMode == eShadeMode.smGouraud)
					v[0].Color = mColor2;
				
				device.DrawUserPrimitives(PrimitiveType.TriangleFan, v, 0, (kNumPts-2));
			}
            return true;
        }

        public override void setFillMode(eFillMode fillMode)
        {
            GraphicsDevice graphicsDevice = MagGame.MyGraphicsManager.GraphicsDevice;

            if (graphicsDevice != null)
            {
                switch (fillMode)
                {
                case eFillMode.fmSolid:
                    graphicsDevice.RenderState.FillMode = FillMode.Solid;
                    break;
                case eFillMode.fmPoint:
                    graphicsDevice.RenderState.FillMode = FillMode.Point;
                    break;
				case eFillMode.fmWireframe:
                    graphicsDevice.RenderState.FillMode = FillMode.WireFrame;
                    break;
                }
            }
        }

        public override bool AccumulateModelTransform(
            Vector3 translation,
            Vector3 scale,
            float rotationRadians,
            Vector3 pivot)
        {
            if (MagGame.MyGraphicsManager.GraphicsDevice == null)
                return false;

            Vector3 z_axis = new Vector3(0, 0, 1);
            m_pMatrixStack.TranslateLocal(translation);
            m_pMatrixStack.TranslateLocal(pivot);
            m_pMatrixStack.RotateAxisLocal(z_axis, rotationRadians);
            m_pMatrixStack.ScaleLocal(scale);
            m_pMatrixStack.TranslateLocal(-pivot);

            MagGame.MyEffect.World = m_pMatrixStack.Top();
            return true;

        }

        public override bool PushModelTransform()
        {
            if (MagGame.MyGraphicsManager.GraphicsDevice == null)
                return false;

            m_pMatrixStack.Push();
            MagGame.MyEffect.World = m_pMatrixStack.Top();
            MagGame.MyEffect.CommitChanges();
            return true;
        }

        public override bool PopModelTransform()
        {
            if (MagGame.MyGraphicsManager.GraphicsDevice == null)
                return false;

            m_pMatrixStack.Pop();
            MagGame.MyEffect.World =  m_pMatrixStack.Top();
            MagGame.MyEffect.CommitChanges();
            return true;
        }

        public override bool InitializeModelTransform()
        {
            if (MagGame.MyGraphicsManager.GraphicsDevice == null)
                return false;

            m_pMatrixStack.LoadIdentity();
            MagGame.MyEffect.World =  m_pMatrixStack.Top();
            MagGame.MyEffect.CommitChanges();
            return true;
        }

        public override bool TransformPoint(ref Vector3 point)
        {
            point = Vector3.Transform(point, m_pMatrixStack.Top());
            return true;
        }

        public override bool EnableBlending(bool on)
        {
            GraphicsDevice d = MagGame.MyGraphicsManager.GraphicsDevice;
            UWB_XNAGraphicsSystem.m_TheAPI.DeactivateTexture();
            if (on)
            {
                d.RenderState.AlphaBlendEnable = true;
                d.RenderState.SourceBlend = Blend.SourceAlpha;
                d.RenderState.DestinationBlend = Blend.InverseSourceAlpha;
                d.RenderState.BlendFunction = BlendFunction.Add;
            }
            else
            {
                d.RenderState.AlphaBlendEnable = false;
            }
            return true;
        }

        public override bool EnableTexture(bool on)
        {
            if (on)
                UWB_XNAGraphicsSystem.m_TheAPI.ActivateTexture(m_TexFileName);
            else
                UWB_XNAGraphicsSystem.m_TheAPI.DeactivateTexture();

            return true;
        }
    }
}
