// University of Washington Bothell Graphics Library
// Modified by: Timothy Chuang
// Based on Kelvin Sung and Steve Baer's D3D Library 
// and Robert Stone's XNA Library version 5
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Storage;
using System.Diagnostics;

namespace MagGame
{
	//This is our custome vertex structure XNA likes to recieve 
	//arrays of structs with vertex information. For simple, unlit
	//drawing; a vertex and color at the vertex will suffice.
    public struct DeviceVertexFormat
    {
        private Vector3 m_Position;
        private Color m_Color;
        private Vector2 m_UV;

        //Format in which this type of vertex will be passed to the graphics hardware
        public static VertexElement[] VertexElements =
        {
            new VertexElement(0, 0, VertexElementFormat.Vector3, VertexElementMethod.Default, 
                              VertexElementUsage.Position, 0),
            new VertexElement(0, sizeof(float)*3, VertexElementFormat.Color, VertexElementMethod.Default, 
                              VertexElementUsage.Color, 0),
            new VertexElement(0, (sizeof(float)*3)+sizeof(uint), VertexElementFormat.Vector2, VertexElementMethod.Default,
                              VertexElementUsage.TextureCoordinate, 0)
        };


        //m_Position accessor and modifier
        public Vector3 Position
        {
            get
            {
                return m_Position;
            }
            set
            {
                m_Position = value;
            }
        }

        //m_Color accessor and modifier
        public Color Color
        {
            get
            {
                return m_Color;
            }
            set
            {
                m_Color = value;
            }
        }

        public float U
        {
            get { return m_UV.X; }
            set { m_UV.X = value; }
        }


        public float V
        {
            get { return m_UV.Y; }
            set { m_UV.Y = value; }
        }

    };

	public class UWB_XNAGraphicsSystem
	{
		private BasicEffect m_BasicEffect;
		private GraphicsDevice m_XNA_GraphicsDevice;
		public static UWB_XNAGraphicsSystem m_TheAPI = new UWB_XNAGraphicsSystem();

		private UWB_XNAGraphicsSystem()
		{
		}



		public GraphicsDevice GetActiveDevice()
		{
			return m_XNA_GraphicsDevice;
		}

		public bool BeginScene(PresentationParameters pp)
		{
			if (m_XNA_GraphicsDevice == null)
				return false;
			m_XNA_GraphicsDevice.Reset(pp);
			m_XNA_GraphicsDevice.VertexDeclaration = new VertexDeclaration(m_XNA_GraphicsDevice, DeviceVertexFormat.VertexElements);
			m_XNA_GraphicsDevice.RenderState.CullMode = CullMode.None;
			m_XNA_GraphicsDevice.RenderState.FillMode = FillMode.Solid;
            // Clear background
            m_XNA_GraphicsDevice.Clear(new Color(new Vector4(0f, .1f, .1f, 0)));

			m_BasicEffect.VertexColorEnabled = true;

			m_BasicEffect.Begin();
			m_BasicEffect.CurrentTechnique.Passes[0].Begin();

			return true;
		}

		public bool EndSceneAndShow(PresentationParameters pp)
		{
			if (m_XNA_GraphicsDevice == null)
				return false;

			m_BasicEffect.CurrentTechnique.Passes[0].End();
			m_BasicEffect.End();
			m_XNA_GraphicsDevice.Present();

			return true;
		}

		public void setViewMatrix(Matrix matrix)
		{
			m_BasicEffect.View = matrix;
            m_BasicEffect.CommitChanges();
		}

        public void setWorldMatrix(Matrix matrix)
        {
            m_BasicEffect.World = matrix;
            m_BasicEffect.CommitChanges();
        }

        // TODO: move this to graphics system class once done testing
        public bool ActivateTexture(string texName)
        {
            Texture2D t = null;
            if (texName == "background")
                t = MagGame.tex_background;
            if (texName == "testtexture")
                t = MagGame.texture;
            if (texName == "tire")
                t = MagGame.tex_tire;
            if (texName == "hardbody")
                t = MagGame.tex_hardbody;
            if (texName == "carbon")
                t = MagGame.tex_carbon;
            if (texName == "arm")
                t = MagGame.tex_shiny;
            if (texName == "armExt")
                t = MagGame.tex_shinymetal;
            if (texName == "chinared")
                t = MagGame.tex_chinared;
            if (texName == "HealthBar")
                t = MagGame.tex_healthbar;
            if (texName == "pausemenu")
                t = MagGame.tex_pausemenu;
            if (texName == "hole")
                t = MagGame.tex_hole;
            if (texName == "splash")
                t = MagGame.tex_splash;
            if (texName == "instructions")
                t = MagGame.tex_instruction;
            if (texName == "redbar")
                t = MagGame.tex_redbar;
            if (texName == "bluebar")
                t = MagGame.tex_bluebar;
            if (texName == "basicshot")
                t = MagGame.tex_normshot;
            if (texName == "background2")
                t = MagGame.tex_background2;
            if (texName == "background3")
                t = MagGame.tex_background3;
            if (texName == "redspiral")
                t = MagGame.tex_redspiral;
            if (texName == "p1wins")
                t = MagGame.tex_p1wins;
            if (texName == "p2wins")
                t = MagGame.tex_p2wins;
            if (texName == "bodyred")
                t = MagGame.tex_bodyred;
            if (texName == "bodyblue")
                t = MagGame.tex_bodyblue;
            if (texName == "yellowtire")
                t = MagGame.tex_yellowtire;
            if (null != t)
            {
                MagGame.MyEffect.Texture = t;
                MagGame.MyEffect.TextureEnabled = true;
                MagGame.MyGraphicsManager.GraphicsDevice.RenderState.PointSpriteEnable = true;
            }
            else
            {
                MagGame.MyEffect.Texture = null;
                MagGame.MyEffect.TextureEnabled = false;
                MagGame.MyGraphicsManager.GraphicsDevice.RenderState.PointSpriteEnable = true;
            }
            MagGame.MyEffect.CommitChanges();
            return true;
        }
        public void DeactivateTexture()
        {
            MagGame.MyEffect.TextureEnabled = false;
            MagGame.MyEffect.CommitChanges();
        }

	}
}